@@ -1,7 +1,7 @@
 /*
  * This file is part of libsidplayfp, a SID player engine.
  *
- * Copyright 2011-2014 Leandro Nini <drfiemost@users.sourceforge.net>
+ * Copyright 2011-2015 Leandro Nini <drfiemost@users.sourceforge.net>
  * Copyright 2007-2010 Antti Lankila
  * Copyright 2001 Simon White
  *
@@ -50,23 +50,31 @@ bool Player::config(const SidConfig &cfg)
         return false;
     }
 
-    uint_least16_t secondSidAddress = cfg.secondSidAddress;
-
     // Only do these if we have a loaded tune
     if (m_tune != 0)
     {
         const SidTuneInfo* tuneInfo = m_tune->getInfo();
 
-        if (tuneInfo->sidChipBase2() != 0)
-            secondSidAddress = tuneInfo->sidChipBase2();
-
         try
         {
             sidRelease();
 
+            std::vector<unsigned int> addresses;
+            const uint_least16_t secondSidAddress = tuneInfo->sidChipBase(1) != 0 ?
+                tuneInfo->sidChipBase(1) :
+                cfg.secondSidAddress;
+            if (secondSidAddress != 0)
+                addresses.push_back(secondSidAddress);
+
+            const uint_least16_t thirdSidAddress = tuneInfo->sidChipBase(2) != 0 ?
+                tuneInfo->sidChipBase(2) :
+                cfg.thirdSidAddress;
+            if (thirdSidAddress != 0)
+                addresses.push_back(thirdSidAddress);
+
             // SID emulation setup (must be performed before the
             // environment setup call)
-            sidCreate(cfg.sidEmulation, cfg.defaultSidModel, cfg.forceSidModel, secondSidAddress);
+            sidCreate(cfg.sidEmulation, cfg.defaultSidModel, cfg.forceSidModel, addresses);
 
             // Determine clock speed
             const c64::model_t model = c64model(cfg.defaultC64Model, cfg.forceC64Model);
@@ -90,9 +98,10 @@ bool Player::config(const SidConfig &cfg)
         }
     }
 
-    m_info.m_channels = secondSidAddress ? 2 : 1;
+    const bool isStereo = cfg.playback == SidConfig::STEREO;
+    m_info.m_channels = isStereo ? 2 : 1;
 
-    m_mixer.setStereo(cfg.playback == SidConfig::STEREO);
+    m_mixer.setStereo(isStereo);
     m_mixer.setVolume(cfg.leftVolume, cfg.rightVolume);
 
     // Update Configuration
@@ -231,15 +240,58 @@ void Player::sidRelease()
     m_mixer.clearSids();
 }
 
+/**
+ * Get the SID model.
+ *
+ * @param sidModel the tune requested model
+ * @param defaultModel the default model
+ * @param forced true if the default model shold be forced in spite of tune model
+ */
+SidConfig::sid_model_t getSidModel(SidTuneInfo::model_t sidModel, SidConfig::sid_model_t defaultModel, bool forced)
+{
+    SidTuneInfo::model_t tuneModel = sidModel;
+
+    // Use preferred speed if forced or if song speed is unknown
+    if (forced || tuneModel == SidTuneInfo::SIDMODEL_UNKNOWN || tuneModel == SidTuneInfo::SIDMODEL_ANY)
+    {
+        switch (defaultModel)
+        {
+        case SidConfig::MOS6581:
+            tuneModel = SidTuneInfo::SIDMODEL_6581;
+            break;
+        case SidConfig::MOS8580:
+            tuneModel = SidTuneInfo::SIDMODEL_8580;
+            break;
+        default:
+            break;
+        }
+    }
+
+    SidConfig::sid_model_t newModel;
+
+    switch (tuneModel)
+    {
+    default:
+    case SidTuneInfo::SIDMODEL_6581:
+        newModel = SidConfig::MOS6581;
+        break;
+    case SidTuneInfo::SIDMODEL_8580:
+        newModel = SidConfig::MOS8580;
+        break;
+    }
+
+    return newModel;
+}
+
 void Player::sidCreate(sidbuilder *builder, SidConfig::sid_model_t defaultModel,
-                        bool forced, const unsigned int secondSidAddresses)
+                        bool forced, const std::vector<unsigned int> &extraSidAddresses)
 {
     if (builder != 0)
     {
         const SidTuneInfo* tuneInfo = m_tune->getInfo();
 
         // Setup base SID
-        const SidConfig::sid_model_t userModel = getModel(tuneInfo->sidModel1(), defaultModel, forced);
+        const SidConfig::sid_model_t userModel = getModel(tuneInfo->sidModel(0), defaultModel, forced);
         sidemu *s = builder->lock(m_c64.getEventScheduler(), userModel);
         if (!builder->getStatus())
         {
@@ -250,18 +302,25 @@ void Player::sidCreate(sidbuilder *builder, SidConfig::sid_model_t defaultModel,
         m_mixer.addSid(s);
 
         // Setup extra SIDs if needed
-        if (secondSidAddresses != 0)
+        if (extraSidAddresses.size() != 0)
         {
             // If bits 6-7 are set to Unknown then the second SID will be set to the same SID
             // model as the first SID.
-            const SidConfig::sid_model_t secondSidModel = getModel(tuneInfo->sidModel2(), userModel, forced);
+            defaultModel = userModel;
 
-            sidemu *s = builder->lock(m_c64.getEventScheduler(), secondSidModel);
+            const unsigned int extraSidChips = extraSidAddresses.size();
 
-            if (!m_c64.addExtraSid(s, secondSidAddresses))
-                throw configError(ERR_UNSUPPORTED_SID_ADDR);
+            for (unsigned int i = 0; i < extraSidChips; i++)
+            {
+                const SidConfig::sid_model_t userModel = getSidModel(tuneInfo->sidModel(i+1), defaultModel, forced);
 
-            m_mixer.addSid(s);
+                sidemu *s = builder->lock(m_c64.getEventScheduler(), userModel);
+
+                if (!m_c64.addExtraSid(s, extraSidAddresses[i]))
+                    throw configError(ERR_UNSUPPORTED_SID_ADDR);
+
+                m_mixer.addSid(s);
+            }
         }
     }
 }
