@@ -1,7 +1,7 @@
 /*
  * This file is part of libsidplayfp, a SID player engine.
  *
- * Copyright 2011-2013 Leandro Nini <drfiemost@users.sourceforge.net>
+ * Copyright 2011-2015 Leandro Nini <drfiemost@users.sourceforge.net>
  * Copyright 2007-2010 Antti Lankila
  * Copyright (C) 2000 Simon White
  *
@@ -37,16 +37,14 @@ class sidemu;
 class Mixer
 {
 public:
-    /// Maximum number of supported SIDs (mono and stereo)
-    static const unsigned int MAX_SIDS = 2;
+    /// Maximum number of supported SIDs
+    static const unsigned int MAX_SIDS = 3;
 
 private:
-    typedef short (Mixer::*mixer_func_t)() const;
+    typedef int_least32_t (Mixer::*mixer_func_t)() const;
 
 public:
-    /**
-     * Maximum allowed volume, must be a power of 2.
-     */
+    /// Maximum allowed volume, must be a power of 2.
     static const int_least32_t VOLUME_MAX = 1024;
 
 private:
@@ -78,17 +76,24 @@ private:
         return oldRandomValue - prevValue;
     }
 
-    short channel1MonoMix() const { return static_cast<short>((m_iSamples[0] + m_iSamples[1]) / 2); }
-    short channel1StereoMix() const { return static_cast<short>(m_iSamples[0]); }
+    // Mono mixing
+    int_least32_t mono_OneChip() const { return m_iSamples[0]; }
+    int_least32_t mono_TwoChips() const { return (m_iSamples[0] + m_iSamples[1]) / 2; }
+    int_least32_t mono_ThreeChips() const { return (m_iSamples[0] + m_iSamples[1] + m_iSamples[2]) / 3; }
+
+    // Stereo mixing
+    int_least32_t stereo_OneChip() const { return m_iSamples[0]; }
+
+    int_least32_t stereo_ch1_TwoChips() const { return m_iSamples[0]; }
+    int_least32_t stereo_ch2_TwoChips() const { return m_iSamples[1]; }
+
+    int_least32_t stereo_ch1_ThreeChips() const { return (m_iSamples[0] + m_iSamples[1]) / 2; }
+    int_least32_t stereo_ch2_ThreeChips() const { return (m_iSamples[1] + m_iSamples[2]) / 2; }
 
-    short channel2FromMonoMix() const { return static_cast<short>(m_iSamples[0]); }
-    short channel2FromStereoMix() const { return static_cast<short>(m_iSamples[1]); }
 
 public:
     /**
      * Create a new mixer.
-     *
-     * @param context event context
      */
     Mixer() :
         oldRandomValue(0),
@@ -96,7 +101,7 @@ public:
         m_sampleCount(0),
         m_stereo(false)
     {
-        m_mix.push_back(&Mixer::channel1MonoMix);
+        m_mix.push_back(&Mixer::mono_OneChip);
     }
 
     /**
@@ -105,7 +110,7 @@ public:
     void doMix();
 
     /**
-     * This clocks the SIDs to the present moment, if they aren't already.
+     * This clocks the SID chips to the present moment, if they aren't already.
      */
     void clockChips();
 
@@ -135,7 +140,7 @@ public:
     void addSid(sidemu *chip);
 
     /**
-     * Get a SID to the mixer.
+     * Get a SID from the mixer.
      *
      * @param i the number of the SID to get
      * @return a pointer to the requested sid emu or 0 if not found
